package com.ningdatech.pmapi.meeting.helper;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.NdDateUtils;
import com.ningdatech.pmapi.meeting.constant.MeetingMsgTemplateConst;
import com.ningdatech.pmapi.meeting.entity.domain.Meeting;
import com.ningdatech.pmapi.meeting.entity.domain.MeetingExpert;
import com.ningdatech.pmapi.organization.model.entity.DingEmployeeInfo;
import com.ningdatech.pmapi.organization.model.entity.DingOrganization;
import com.ningdatech.pmapi.organization.service.IDingEmployeeInfoService;
import com.ningdatech.pmapi.organization.service.IDingOrganizationService;
import com.ningdatech.pmapi.staging.enums.MsgTypeEnum;
import com.ningdatech.pmapi.staging.service.INdWorkNoticeStagingService;
import com.ningdatech.pmapi.sys.model.entity.Notify;
import com.ningdatech.pmapi.sys.service.INotifyService;
import com.ningdatech.pmapi.todocenter.bean.entity.WorkNoticeInfo;
import com.ningdatech.pmapi.user.entity.UserInfo;
import com.ningdatech.pmapi.user.service.IUserInfoService;
import com.ningdatech.yxt.model.cmd.SendSmsCmd.SendSmsContext;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * MeetingMsgHelper
 * </p>
 *
 * @author WendyYang
 * @since 2023/4/20
 **/
@Component
@AllArgsConstructor
public class MeetingMsgHelper {

    private final IUserInfoService userInfoService;
    private final YxtCallOrSmsHelper yxtCallOrSmsHelper;
    private final INdWorkNoticeStagingService workNoticeStagingService;
    private final IDingEmployeeInfoService dingEmployeeInfoService;
    private final IDingOrganizationService dingOrganizationService;
    private final INotifyService notifyService;

    private Notify getNotify(Long userId, String msg, Map<String, Object> extraPara) {
        Notify notify = new Notify();
        notify.setUserId(userId);
        notify.setContent(msg);
        notify.setReaded(Boolean.FALSE);
        notify.setCreateTime(LocalDateTime.now());
        String extraJson = JSON.toJSONString(extraPara);
        notify.setExtraInfo(extraJson);
        return notify;
    }

    private WorkNoticeInfo getSendWorkNoticeInfo(Long accountId) {
        WorkNoticeInfo workNoticeInfo = new WorkNoticeInfo();
        workNoticeInfo.setAccountId(accountId);
        // 根据浙政钉用户ID获取部门code
        DingEmployeeInfo employeeInfo = dingEmployeeInfoService.getOne(Wrappers.lambdaQuery(DingEmployeeInfo.class)
                .eq(DingEmployeeInfo::getAccountId, accountId)
                .eq(DingEmployeeInfo::getMainJob, String.valueOf(Boolean.TRUE))
                .last("limit 1"));
        String organizationCode = employeeInfo.getOrganizationCode();
        workNoticeInfo.setOrganizationCode(organizationCode);
        // 根据部门code获取部门名称
        DingOrganization dingOrganization = dingOrganizationService.getOne(Wrappers.lambdaQuery(DingOrganization.class)
                .eq(DingOrganization::getOrganizationCode, organizationCode));
        String organizationName = dingOrganization.getOrganizationName();
        workNoticeInfo.setOrganizationName(organizationName);
        // 构建唯一的消息ID
        String bizMsgId = "ZD_WORK_NOTICE_" + StrUtil.UNDERLINE + organizationCode + StrUtil.UNDERLINE
                + organizationName + accountId + StrUtil.UNDERLINE + System.currentTimeMillis();
        workNoticeInfo.setBizMsgId(bizMsgId);
        String receiverUserId = String.valueOf(accountId);
        workNoticeInfo.setReceiverUserId(receiverUserId);
        return workNoticeInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    public void sendInviteStopMsg(Long userId, Long meetingId, String meetingName) {
        UserInfo info = userInfoService.getById(userId);
        String msgContent = String.format(MeetingMsgTemplateConst.INVITE_END, meetingName);
        // 音信通消息
        SendSmsContext yxtContent = new SendSmsContext();
        yxtContent.setContent(msgContent);
        yxtContent.setReceiveNumber(info.getMobile());
        yxtCallOrSmsHelper.sendSms(yxtContent);
        // 发送工作通知
        if (info.getAccountId() != null) {
            WorkNoticeInfo swn = getSendWorkNoticeInfo(info.getAccountId());
            swn.setMsg(msgContent);
            workNoticeStagingService.addByWorkNotice(swn, MsgTypeEnum.REVIEW_MEETING);
            Map<String, Object> map = new HashMap<>();
            map.put("meetingId", meetingId);
            Notify notify = getNotify(userId, msgContent, map);
            notifyService.save(notify);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void sendConfirmedRosterMsg(List<MeetingExpert> experts, Meeting meeting) {
        List<Long> userIds = CollUtils.fieldList(experts, MeetingExpert::getExpertId);
        List<UserInfo> userInfos = userInfoService.listByIds(userIds);
        Map<Long, UserInfo> userMap = CollUtils.listToMap(userInfos, UserInfo::getId);
        String sTime = meeting.getStartTime().format(DatePattern.NORM_DATETIME_MINUTE_FORMATTER);
        String eTime = meeting.getEndTime().format(DatePattern.NORM_DATETIME_MINUTE_FORMATTER);
        String meetingTime = sTime + " - " + eTime;
        List<SendSmsContext> yxtContents = new ArrayList<>();
        List<Notify> notifies = new ArrayList<>();
        List<WorkNoticeInfo> workingNotices = new ArrayList<>();
        experts.forEach(w -> {
            String msgContent = String.format(MeetingMsgTemplateConst.CONFIRMED_ROSTER,
                    w.getExpertName(), w.getCreateOn(), meetingTime, meeting.getMeetingAddress(),
                    meeting.getConnecter(), meeting.getContact());
            // 音信通消息
            SendSmsContext yxtContent = new SendSmsContext();
            yxtContent.setContent(msgContent);
            yxtContent.setReceiveNumber(w.getMobile());
            yxtContents.add(yxtContent);
            UserInfo info = userMap.get(w.getExpertId());
            // 发送工作通知
            if (info.getAccountId() != null) {
                WorkNoticeInfo swn = getSendWorkNoticeInfo(info.getAccountId());
                swn.setMsg(msgContent);
                workingNotices.add(swn);
                Map<String, Object> map = new HashMap<>();
                map.put("meetingId", meeting.getId());
                Notify notify = getNotify(info.getId(), msgContent, map);
                notifies.add(notify);
            }
        });
        notifyService.saveBatch(notifies);
        yxtCallOrSmsHelper.sendSms(yxtContents);
        workNoticeStagingService.addByWorkNotice(workingNotices, MsgTypeEnum.EXPERT_REVIEW);
    }


}
